@@ -8,6 +8,11 @@ Check the license on the LICENSE file
 import requests
 import json
 from . import exceptions
+try:
+    from urllib import quote_plus
+except ImportError:
+    from urllib.parse import quote_plus
+    basestring = str
 
 
 class Gitlab(object):
@@ -183,13 +188,12 @@ class Gitlab(object):
         else:
             return False
 
-    def getsshkeys(self, page=1, per_page=20):
+    def getsshkeys(self):
         """Gets all the ssh keys for the current user
 
         :return: a dictionary with the lists
         """
-        data = {'page': page, 'per_page': per_page}
-        request = requests.get(self.keys_url, headers=self.headers, params=data,
+        request = requests.get(self.keys_url, headers=self.headers,
                                verify=self.verify_ssl)
         if request.status_code == 200:
             return json.loads(request.content.decode("utf-8"))
@@ -301,17 +305,18 @@ class Gitlab(object):
             return False
 
     def getproject(self, project_id):
-        """Get info for a project identified by id
+        """Get info for a project identified by id or namespace/project_name
 
-        :param project_id: id of the project
+        :param project_id: id or namespace/project_name of the project
         :return: False if not found, a dictionary if found
         """
+        if isinstance(project_id, basestring):
+            project_id = quote_plus(project_id)
         request = requests.get("{0}/{1}".format(self.projects_url, project_id),
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
             return json.loads(request.content.decode("utf-8"))
         else:
-
             return False
 
     def getprojectevents(self, project_id, page=1, per_page=20):
@@ -518,14 +523,18 @@ class Gitlab(object):
 
             return False
 
-    def addprojecthook(self, project_id, url):
-        """Add a hook to a project
-
-        :param project_id: project id
+    def addprojecthook(self, project_id, url, push=False, issues=False, merge_requests=False, tag_push=False):
+        """
+        add a hook to a project
+        :param id_: project id
         :param url: url of the hook
         :return: True if success
         """
         data = {"id": project_id, "url": url}
+        data['push_events'] = int(bool(push))
+        data['issues_events'] = int(bool(issues))
+        data['merge_requests_events'] = int(bool(merge_requests))
+        data['tag_push_events'] = int(bool(tag_push))
         request = requests.post("{0}/{1}/hooks".format(self.projects_url, project_id),
                                 headers=self.headers, data=data, verify=self.verify_ssl)
         if request.status_code == 201:
@@ -533,17 +542,20 @@ class Gitlab(object):
         else:
             return False
 
-    def editprojecthook(self, project_id, hook_id, url):
-        """Edit an existing hook from a project
-
-        :param project_id: project id
+    def editprojecthook(self, project_id, hook_id, url, push=False,
+            issues=False, merge_requests=False, tag_push=False):
+        """
+        edit an existing hook from a project
+        :param id_: project id
         :param hook_id: hook id
         :param url: the new url
-        :param sudo: do the request as another user
         :return: True if success
         """
         data = {"id": project_id, "hook_id": hook_id, "url": url}
-
+        data['push_events'] = int(bool(push))
+        data['issues_events'] = int(bool(issues))
+        data['merge_requests_events'] = int(bool(merge_requests))
+        data['tag_push_events'] = int(bool(tag_push))
         request = requests.put("{0}/{1}/hooks/{2}".format(self.projects_url, project_id, hook_id),
                                headers=self.headers, data=data, verify=self.verify_ssl)
         if request.status_code == 200:
@@ -619,14 +631,13 @@ class Gitlab(object):
         else:
             return False
 
-    def getbranches(self, project_id, page=1, per_page=20):
+    def getbranches(self, project_id):
         """List all the branches from a project
 
         :param project_id: project id
         :return: the branches
         """
-        data = {'page': page, 'per_page': per_page}
-        request = requests.get("{0}/{1}/repository/branches".format(self.projects_url, project_id), params=data,
+        request = requests.get("{0}/{1}/repository/branches".format(self.projects_url, project_id),
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
             return json.loads(request.content.decode("utf-8"))
@@ -910,14 +921,13 @@ class Gitlab(object):
 
             return False
 
-    def getdeploykeys(self, project_id, page=1, per_page=20):
+    def getdeploykeys(self, project_id):
         """Get a list of a project's deploy keys.
 
         :param project_id: project id
         :return: the keys in a dictionary if success, false if not
         """
-        data = {'page': page, 'per_page': per_page}
-        request = requests.get("{0}/{1}/keys".format(self.projects_url, project_id), params=data,
+        request = requests.get("{0}/{1}/keys".format(self.projects_url, project_id),
                                headers=self.headers, verify=self.verify_ssl)
         if request.status_code == 200:
             return json.loads(request.content.decode("utf-8"))
@@ -1850,3 +1860,26 @@ class Gitlab(object):
             return json.loads(request.content.decode("utf-8"))
         else:
             return False
+
+    @staticmethod
+    def getall(fn, *args, **kwargs):
+        """Auto-iterate over the paginated results of various methods of the API.
+        Pass the GitLabAPI method as the first argument, followed by the
+        other parameters as normal. Include `page` to determine first page to poll.
+        Remaining kwargs are passed on to the called method, including `per_page`.
+
+        :param fn: Actual method to call
+        :param *args: Positional arguments to actual method
+        :param page: Optional, page number to start at, defaults to 1
+        :param **kwargs: Keyword arguments to actual method
+        :return: Yields each item in the result until exhausted, and then
+        implicit StopIteration; or no elements if error
+        """
+        page = kwargs.pop('page', 1)
+        while True:
+            results = fn(*args, page=page, **kwargs)
+            if not results:
+                break
+            for x in results:
+                yield x
+            page += 1
